; Bootloader and System routines - MAIN UNIT
; EHBASIC extensions for T65 Core on FPGA by C. Meyer 3/2009

	.START Reset
	.ROM_AREA $C000, $FFFF
	.OPT	Proc6502, CaseInsensitive

RealWorld	= 1	; luft auch im Simulator wenn = 0
GetRAMsize 	= 0	; RAM-Abfrage bei Start wenn = 1
LCD320		= 0	; Version fr LCD 320 x 240, 40 Zeichen / 20 zeilen

	.INCLUDE "defines_cm.asm"	; HW- und ZeroPage-Belegung

	*=$BD00
	.INCLUDE "renumber_cm.asm"	; HW- und ZeroPage-Belegung

	*=ROM_start

;Jump Table of important ROM routines	

	JMP 	Reset		; $C000, Cold Start
	JMP 	LAB_WARM	; $C003, do EhBASIC warm start
	JMP 	ClearVars	; $C006, Reset Vars, String Space etc.
	
	JMP 	AccToBank	; $C009, Memory Bank Management
	JMP 	BankToAcc  	; $C00C 

	JMP 	TickerDelay	; $C00F, VSync delay * Acc
	JMP 	Getkey		; der auf ZP $03, kann verbogen werden
	JMP 	Waitkey  		
	JMP 	WriteScreen		
	JMP 	WriteAcc		
	JMP 	ReceiveByte	; ber COMCH
	JMP 	SendByte	; ber COMCH
	JMP 	LAB_SCGB	; wichtig fr CALL Erweiterungen: ",<bytevar>"
; LAB_SCGI, wichtig fr CALL Erweiterungen: ",<intvar>", Ergebnis in Itempl/Itemph
	JSR	LAB_SCOMMA	; scan for "," , else do syntax error then warm start
; LAB_EVNM, wichtig fr CALL Erweiterungen, wie vor, jedoch ohne Komma
	JSR	LAB_EVNM	; evaluate expression and check is numeric,					; else do type mismatch
	JMP	LAB_F2FX	; save integer part of FAC1 in temporary integer

	.INCLUDE "basic_cm.asm"
	.include "monitor_cm.asm"

;#####################################################
;###             Memory Bank Access                ###
;#####################################################
	
; for Acc move from/to (BankPtrL),Y in Bank (AccessBank)
; BankPtrL = $52
; BankPtrH = $53
; AccessBank = $46, 70 dez.
; Benutzt X-Reg., Y als zus. Pointer (=0 wenn nicht gebraucht)

AccToBank:	
	LDX AccessBank
	STX Bank
	STA (BankPtr),Y
	JMP BankRestore
BankToAcc:
	LDX AccessBank
	STX Bank
	LDA (BankPtr),Y
BankRestore:
	LDX CurrBank
	STX Bank
	RTS


	.IF RealWorld

;#####################################################
;###          einfaches Terminalprogramm           ###
;#####################################################

Terminal:
	LDA #0	; Keyboard und Screen EIN
	STA Omode	; kein LABOUT
	STA Imode	; kein READ
	STA CapsFlg
	JSR cls
	JSR LAB_FlushRx
TermRelease:
	JSR TermWrite
	LDA PS2code
	BNE TermRelease	; PS2code 0 = keine Taste
TermScan:
	JSR TermWrite
	LDA #$80	; ACK
	STA Tx0ACK
	LDA PS2code
	BEQ TermScan
	JSR PS2scan
	JSR UpCaseCaps
	CMP #$1B	; ESC
	BEQ TermEnd
	JSR SendByte
	JSR WriteScreen	; Local Echo
	CMP #$0D	; CR? LF anfgen
	BNE TermRelease
	LDA #$0A
	JSR WriteScreen
TermScanEnd:
	JMP TermRelease
TermEnd:
	LDA IRQack	; IRQ-Latch lschen
	LDA IRQclr
	LDA #0	
	STA IRQena
	LDA #$20	; Grobuchstaben in BASIC
	STA CapsFlg
TermEndRel:
	LDA PS2code	; Taste gedrckt?
	BNE TermEndRel	; Warten bis losgelassen
	LDA #$80	; FlowControl einschalten
	STA Tx0ACK	; aber auch setzen
	LDA #$00	; wichtig: Break-Flag, Dezimal-Flag lschen
	PHA		
	PLP
	RTS
	
TermWrite:
	JSR ReceiveByte
	BCC TermNowrite0	; nix da
	JSR WriteScreen
	CMP #$0D	; CR? LF anfgen
	BNE TermNowrite0
	LDA #$0A
	JSR WriteScreen
TermNowrite0:
	RTS

;############################################################
;#	 Zeichen-Ausgabe auf Bildschirm oder COM	    #
;############################################################

; Kommunikationsroutinen
	

omode_stx:		; sende STX (Start Text), warte auf ACK
	LDA #$80
	STA Omode
	LDA #2		; CTRL-B als Startzeichen senden
	JSR SendByte	; 
	STY ysave
	LDA #4	
	JSR TickerDelay	; Verarbeitung abwarten
	LDY ysave
	RTS
	
omode_etx:
	LDA #$00
	STA Omode
	LDA #3		; CTRL-C/ETX als Ende-Zeichen-senden
	JMP SendByte	; nicht auf Screen, sondern Abschlusszeichen seriell

WriteAcc:		; Zeichen in Acc ausgeben
	CMP #$02	; STX? Omode auf AVR-COM/OptoBusCore, $02 senden
	BEQ omode_stx
	CMP #$03	; ETX? Omode auf Screen
	BEQ omode_etx	
	BIT Omode	; wenn $80, Ausgabe auf COM
	BPL WriteScreen
;	BVS WriteStatus	; Fehlermeldungen landen hier mit omode=$40

SendByte:		; Zeichen auf AVR-COM ausgeben
	STA asave
	STY ysave
	LDA COMCH
	BEQ SendByte0

SendByte1:		; SerialCore
	BIT UART1TxS
	BVS SendByte1	; warte bis Buffer nicht mehr voll
	LDA asave
	STA UART1D	; ausgeben
	LDY #0
Ydelay:		; 102 us Pause pro Byte
	DEY
	BNE Ydelay
	LDY ysave
	RTS
		
SendByte0:		; AVR COMport SPI
	BIT Tx0ACK
	BPL SendByte0	; noch kein ACK empfangen?
SendByte0weiter:
	LDA asave
	STA COMport
	LDY #$01	; Byte ausgeben, F_INT kurz triggern
	STY IRQena
	DEY
	STY IRQena	; ist 0
	JSR Ydelay
	CMP #$0D	; Acc=CR?
	BNE SendByte0exit
	LDA #0
	STA Tx0ACK	; ACK lschen nach CR
SendByte0exit:
	LDY ysave
	LDA asave		
	RTS

WriteStatus:		; Fehlermeldungen landen hier mit omode=$40

WriteScreen:		; Akku auf Screen, aktualisiert Cursor X/Y
	STA asave
	STX xsave
	STY ysave
; I/O-Steuerzeichen behandeln
	CMP #$20	; writable?
	BPL writeChar
	CMP #$0D	; CR
	BEQ cr_jmp
	CMP #$0A	; Linefeed
	BEQ lf_jmp
	CMP #$03	; ETX? Omode auf Screen
	BEQ omode_etx
	CMP #$08	; Backspace?
	BEQ del_jmp
	CMP #$0B	; Vertical Tab?
	BEQ vt
	CMP #$09	; Horizontal Tab?
	BEQ ht
	CMP #$0C	; Clear Screen
	BEQ cls_jmp
	CMP #$1B	; ESC
	BEQ restore
	CMP #$1F	; Home Cursor
	BEQ hc_jmp
	CMP #$20
	BMI restore	; andere Control-Codes unbercksichtigt
	
writeChar:
	;CMP #$7F	; Delete?
	;BEQ del_jmp
	LDX Color
	STX ColorP
	LDX InsFlg
	BEQ writeChar1
	JSR Insert
writeChar1:
	LDX row
	JSR BaseCalc
	LDY col
	STA (ptrdest),Y
	INY
	STY col
	DEY
	CPY TextWinRight	; #colstotal	; Ende der Zeile?
	BNE restore		; nein
	LDY TextWinLeft		; #0		; Zeilenanfang
	STY col
	JMP lf

cr_jmp:
	JMP cr
del_jmp:
	JMP del
cls_jmp:
	JMP cls
lf_jmp:
	JMP lf
bs_jmp:
	JMP bs
hc_jmp:
	JMP hc

restore:
	LDA col
	STA cursx	; fr HW-Cursor
	LDA row
	STA cursy	; fr HW-Cursor
restoreRegs:
	LDX xsave
	LDY ysave
	LDA asave
	RTS	

vt:	LDX row
	CPX TextWinTop	; #rowstart
	BEQ restore	; schon ganz oben?
	DEX
	STX row
	JMP restore

bs:	LDX col
	CPX TextWinLeft
	BEQ restore	; nicht unter 0
	DEX
	STX col
	JMP restore
	
ht:	LDX col
	INX
	CPX TextWinRight	; #colstotal-1
	BEQ restore	; nicht ber Ende der Zeile
	STX col
	JMP restore

lf:	LDX row
	INX
	STX row
	DEX
	CPX TextWinBtm	; #rowstotal
	BEQ scroll
	JMP restore

hc:	LDX TextWinTop	; #rowstart	; Home Cursor
	STX row
	LDY TextWinLeft	; #0
	STY col
	JMP restore

cr:	LDX TextWinLeft	; #0
	STX col
	LDX #0
	STX InsFlg
	JSR InsertStatus
cr_end:
	JMP restore

del:	LDX col
	BEQ del_end	; nicht unter 0
	DEX
	STX col
	LDX row
	JSR BaseCalc
	LDA #$20	; mit Leerzeichen lschen
	LDY col
	STA (ptrdest),Y
del_end:
	JMP restore

; Screen-Speicherplatz 1. Spalte ber HW-Multiplier berechnen 
; und in PtrDest speichern. Zeile in X. Verndert kein Register.

BaseCalc:
	PHA
	LDA #0
	STA MultA_hi
	STA MultB_hi
	LDA #colstotal	; Zeichen pro Zeile
	STA MultB
	TXA		; Zeilennummer wurde in X bergeben
	STA MultA
	LDA MultResult
	STA ptrdest
	LDA MultResult+1
	CLC
	ADC #>VRAM
	STA ptrdest+1
	PLA
	RTS
	
; Zeichen in A in Zeile X und Spalte Y schreiben
WriteCharXY:
	JSR BaseCalc
	STA (ptrdest),Y
	RTS

scroll:
	LDX TextWinTop	; #rowstart+1
	INX
	STX work
linecopy:
	LDY TextWinLeft	;#0  80 Zeichen kopieren
	DEY
	LDX work
	JSR BaseCalc
	LDA ptrdest
	STA ptrsrc	; wird Quelladresse
	LDA ptrdest+1
	STA ptrsrc+1
	DEX
	JSR BaseCalc
lineloop:	
	INY
	LDA (ptrsrc),Y
	STA (ptrdest),Y
	CPY TextWinRight	;#colstotal	; Ende der Zeile?
	BNE lineloop
	LDA work
	INC work
	CMP TextWinBtm	; #rowstotal	; letzte Zeile?
	BNE linecopy
	LDX TextWinBtm	; #rowstotal-1
	JSR lineclr
	STX row
	JMP restore

lineclr:			;Lscht Zeile in X
	JSR BaseCalc
	LDY TextWinLeft		; #0
	DEY
	LDA #$20		; Leerzeichen
lineclrloop:	
	INY
	STA (ptrdest),Y
	CPY TextWinRight	; #colstotal	; Ende der Zeile?
	BNE lineclrloop
	RTS
		
SetFullScreen:
;rowstart  = 2	; 
;rowstotal = 39	; 
;colstotal = 80	; 
	LDA #$00
	STA TextWinLeft
	STA col
	STA cursx
	LDA #colstotal-1
	STA TextWinRight
	LDA #rowstart
	STA TextWinTop
	LDA #rowstotal-1
	STA TextWinBtm
	RTS

		
ClearFullScreen:
	STA asave
	STX xsave
	STY ysave

	JSR SetFullScreen
	LDX #rowstotal		; Mll in letzter Zeile+1
	JSR lineclr
	
	LDA #$02		; CLS von Tastatur oder Reset immer auf grn
	STA Color
	LDA #$00
	STA GFrameC
	JSR FrameClear		; Grafik lschen
cls:	LDY #0
	LDX #0
	.IF LCD320
	LDA #$01
	.ELSE
	LDA #$07
	.ENDIF
	
	STA ColorP
statusloop1:		; Statuszeile ausgeben
	LDA statusline1,y
	BEQ statusloop2	; String-Ende?
	JSR writecharxy
	INY 
	BNE statusloop1	; max. 255 Zeichen
statusloop2:		; Rest der Statuszeile lschen
	LDA #$20	
statusloop3:
	JSR writecharxy
	INY 
	CPY #colstotal	; Ende der Zeile?
	BNE statusloop3	
	
	.IF LCD320
	.ELSE
statusbar:
	LDY #0
	INX
	LDA #202	; code page 437 "-"
statusbarloop:
	JSR writecharxy
	INY
	CPY #colstotal
	BNE statusbarloop
	.ENDIF
	
	JSR ClearWindow
	JSR CapsStatus
	JSR InsertStatus
	JSR COMstatus
	JSR BankStatus
	JSR ErrStatus
	JMP restore

ClearWindow:
	LDA Color	; letzte Grundfarbe
	STA ColorP
	LDY #$00
	LDA #$20
	LDX TextWinTop	; #rowstart 
clsloop:
	JSR lineclr
	INX
	CPX TextWinBtm	; #rowstotal+1
	BNE clsloop
	JSR lineclr
	LDY TextWinLeft
	STY col
	LDX TextWinTop	; #rowstart
	STX row
	RTS
			
Insert:			; Bildschirmzeile ab Cursor eins nach rechts
	PHA
	LDX row
	JSR BaseCalc
	LDY TextWinRight	; #colstotal-1	; 80/40 Zeichen
InsLoop:
	DEY	
	LDA (ptrdest),Y
	INY
	STA (ptrdest),Y
	DEY
	CPY col		; An Cursorposition?
	BNE InsLoop
	LDA #$20
	LDY col
	STA (ptrdest),Y
	PLA
	RTS
	
 	.INCLUDE "basic_kbd.asm"	; Tastatur-Scancodes

;############################################################################
;#	            Tastaturabfrage und PS/2-Konvertierung                  #
;############################################################################


GetKeySPI:
	LDA #0
	LDY KbdPtr		; prfen, ob SPI-"Taste" eingetroffen
	CPY iKbdPtr
	BEQ GetKeySPInone	; wenn nix mehr da
	INY
	STY KbdPtr
	LDA KbdBuf,Y
	RTS
GetKeySPInone:
	CMP #0
	RTS

; Taste abfragen: wartet bis Taste gedrckt
; ASCII-Tastenwert in A und Carry "1"

WaitKey:
	JSR GetKey
	BEQ WaitKey
	RTS
	
; Taste abfragen: wartet nur auf Loslassen, fhrt Autorepeat aus
; keine Taste gedckt: A=0, Carry "0"
; sonst ASCII-Tastenwert in A und Carry "1"

Getkey:
	STY ykey
	STX xkey
	JSR GetKeySPI
	BNE GetKeyOK

	LDA lastkey
	BNE Getkeywait
	JSR PS2scan
	CMP #0	; WICHTIG! sonst klappt der 0-Vergleich in EhBASIC nicht
	BEQ GetkeyNone
	STA lastkey
	JSR UpCaseConvert
GetKeyOK:
	LDY #KeyEventCode
	STY peekEvent

	LDY ykey
	LDX xkey
	CMP #0	; WICHTIG! sonst klappt der 0-Vergleich in EhBASIC nicht
	SEC
	RTS
GetkeyNone:
	LDY ykey
	LDX xkey
	LDA #0
	sta lastkey
	CLC
	RTS
		
GetkeyWait:		; auf Loslassen warten
	JSR PS2scan
	CMP lastkey	; Taste gewechselt, loslassen simulieren
	BNE GetKeyTimeout
	STA lastkey
	JSR RepeatTimer
	BEQ GetKeyTimeout	; Timer abgelaufen
	LDA lastkey
	BNE Getkeywait
GetKeyTimeout:
	LDY ykey
	LDX xkey
	LDA #0
	STA lastkey
	CLC
	RTS

	LDA PS2code	; E0-Codes haben hier Bit 7 gesetzt, Rest wie letztes Byte
	TAY
	LDA PS2toASCII,Y	; Default: Unshifted


; Byte in Acc vom FPGA-Modul SPI, von SerialCore oder OptoBusCore
; setzt Carry=1 wenn Byte empfangen, sonst Carry=0 und Acc=0

ReceiveByte:
	STY ysave
; von AVRCOM- oder UART-IRQ im Buffer abgelegt
	LDY RxPtr	; prfen, ob etwas empfangen wurde
	CPY iRxPtr
	BEQ ReceiveByteExitNone	; nix da
	INY
	STY RxPtr
	LDA RxBuf,Y

ReceiveByteOK:
	LDY ysave
	CMP #0
	SEC
	RTS

ReceiveByteExitNone:
	LDY ysave
ReceiveStringError:
	LDA #0
	CLC
	RTS
	

; Universelle String-Input-Routine, behandelt CR/LF und LabReqIgnoreChr
; als Ende-Zeichen
; Fr LABINP-Antwort vom Modul setzt sie
; HashSignIdx	= $0716	; Position nach "#" in Ibuffs
; EqualSignIdx	= $0717 ; Position nach "=" in Ibuffs
; Setzt ein (von INPUT entferntes) <"> an den Ibuffs-Anfang $220

ReceiveString:
	LDA 	#15		; Timeout 0,25s (60Hz/6)
	STA 	TimeOut
	LDX	#0		; Pointer fr Ibuffs
	STX	HashSignIdx
	STX	EqualSignIdx
	STX	ColonSignIdx
	LDA	#'"'		; String-Input erwnscht, also mit "-Anfang simulieren
	STA	Ibuffs
	STA	Srchc
	INX
ReceiveString1:
	LDA	#0
	STA	Ibuffs,X	; Mit Ende-Null vorbelegen
	LDA	TimeOut
	BEQ	ReceiveStringError
	JSR	ReceiveByte
	BCC	ReceiveString1
	CMP	#$0A		; evt. brig von der letzten Antwort, ignorieren
	BEQ	ReceiveString1
	CMP 	#$0D		; CR?
	BEQ 	ReceiveStringEnded
	CMP 	LabReqIgnoreChr
	BEQ 	ReceiveStringRemaining
	CMP 	#'#'		; Startzeichen der Antwort?
	BNE 	ReceiveString2
	STX	HashSignIdx
	INC	HashSignIdx
ReceiveString2:
	CMP 	#':'		; Ergebnis der Antwort?
	BNE 	ReceiveString3
	STX	ColonSignIdx
	INC	ColonSignIdx
ReceiveString3:
	CMP 	#'='		; Ergebnis der Antwort?
	BNE 	ReceiveString4
	STX	EqualSignIdx
	INC	EqualSignIdx
ReceiveString4:
	CPX	#Ibuffe-Ibuffs	; compare character count with max
	BCS	ReceiveStringError	; exit if buffer full, Error!
	STA	Ibuffs,X	; else store in buffer
	INX
	BNE	ReceiveString1	; immer!
	BEQ	ReceiveStringError
	
ReceiveStringEnded:
	SEC
	RTS
	
ReceiveStringRemaining:
; Rest nach LabReqIgnoreChr abrufen (aber nicht speichern) bis CR kommt
	LDA	TimeOut
	BEQ	ReceiveStringError
	JSR	ReceiveByte
	BCC	ReceiveStringRemaining
	CMP 	#$0D		; CR?
	BNE	ReceiveStringRemaining
	LDX	#'X'-'0'		; "Extra Ignored (Rest of Line)"
	STX	LabErrFlags
	JSR 	ErrStatus
	JMP 	ReceiveStringEnded


TickerDelay:	; Timer-Wert in Acc, 60 => 1s
	STA	TimeOut
TickerDelayLoop:
	LDA	TimeOut	; NMI-Timer
	BNE	TickerDelayLoop
	RTS
	
	.ELSE

;############################################################################
; Testroutinen fr Simulator
; setup for the 6502 simulator environment

IO_AREA	= $8000		; set I/O area for this monitor
ACIAsimwr	= IO_AREA+$01	; simulated ACIA write port
ACIAsimrd	= IO_AREA+$04	; simulated ACIA read port

WriteAcc:		; Zeichen in Acc ausgeben
	BIT Omode	; wenn $80, Ausgabe auf COM
	BPL WriteScreen
SendByte:		; ersetzt PutByte
	PHA
	LDA #'^'
	STA ACIAsimwr	; save byte to simulated ACIA
	PLA
WriteScreen:
	STA ACIAsimwr	; save byte to simulated ACIA
	;JSR WriteScreen
	RTS

; byte in from simulated ACIA
ReceiveVal:
ReceiveByte:
GetKey:
	LDA ACIAsimrd	; get byte from simulated ACIA
	BEQ LAB_nobyw	; branch if no byte waiting
	SEC		; flag byte received
	RTS
LAB_nobyw
	CLC		; flag no byte received
	RTS
WaitKey:
	LDA ACIAsimrd	; get byte from simulated ACIA
	BEQ WaitKey	; branch if no byte waiting
	RTS
CapsToggle:
	RTS
Terminal:
	JSR Waitkey
	RTS
StartTTY:
BankStatus:
ClearFullScreen:
ClearWindow:
COMstatus:
writecharxy:
SetFullScreen:
TickerDelay:	; Timer-Wert in Acc, 60 => 1s
	RTS

	.ENDIF	



;############################################################################
;#                   OS-Routinen, Hilfsroutinen COM                         #
;############################################################################
	
VSwait:	; Delay 16ms
	BIT syncFlgs	; In flyback ?
	BPL VSwait	; ..yes, wait until finished
VSwait2:		; VSync Flyback
	BIT syncFlgs	; in flyback ?
	BMI VSwait2	; ..no, wait for flyback
	RTS
	
; does BASIC Timeout error
AVRCOM_ERR
	LDX	#$1E			; error code $1E ("Timeout" error)
	JMP	LAB_XERR		; do error #X, then warm start

	.IF RealWorld

	
;Warten bis Antwort vom AVR (intern) eintrifft	
WriteResponse:
	JSR	ReceiveString
	BCC	WriteResponseExit
	JSR	WriteIbuffS
	JMP	WriteResponse		; nochmal versuchen
WriteResponseExit:
	RTS
		
; prfen, ob es eine Antwort auf den Schirm zu schreiben gibt
GetResponse:
	JSR	ReceiveString 		;
	BCC	AVRCOM_ERR		; wenn timeout-Fehler
GetResponse1:
	CLC
WriteIbuffS:
	LDA	#<IBuffS+1		; Adresse des Strings +1 wg. "
	LDY	#>IBuffS
	JSR	LAB_18C3		; print null terminated string from memory
	JMP	LAB_CRLF		; print CR/LF and return
	
	
	.ELSE
WriteResponse:
GetResponse:
	RTS
	.ENDIF

; allg. Ausgabe an AVR

; Hilfsroutine zur Screen-Ausgabe eines Strings nach JSR WriteMsg
WriteMsg:		; Put Msg String Routine
	PLA		; Get the return address LSB from stack
	STA Cmdptr	; ..into w/s
	PLA		; Get the return address MSB from stack
	STA Cmdptr+1	; ..into w/s
WriteMsgChar:
	LDY #$0
	INC Cmdptr	; Point to the next byte of embedded text
	BNE WriteMsgNext
	INC Cmdptr+1	; Increment MSB text pointer if carried over
WriteMsgNext:
	LDA (Cmdptr),Y	; Get the next character of embedded text
	BMI WriteMsgExit; ..reached terminator - execute from here
	JSR WriteScreen	; Send character to Screen
	JMP WriteMsgChar; ..and go for the next character
WriteMsgExit:
	JMP (Cmdptr)	; Continue execution from the negative byte

; Hilfsroutine zur COM-Ausgabe eines Strings nach JSR PutMsg
SendMsg:		; Put Msg String Routine
	PLA		; Get the return address LSB from stack
	STA Cmdptr	; ..into w/s
	PLA		; Get the return address MSB from stack
	STA Cmdptr+1	; ..into w/s
SendMsgChar:
	LDY #$0
	INC Cmdptr	; Point to the next byte of embedded text
	BNE SendMsgNext
	INC Cmdptr+1	; Increment MSB text pointer if carried over
SendMsgNext:
	LDA (Cmdptr),Y	; Get the next character of embedded text
	BMI SendMsgExit	; ..reached terminator - execute from here
	JSR SendByte	; Send character to AVR
	JMP SendMsgChar	; ..and go for the next character
SendMsgExit:
	JMP (Cmdptr)	; Continue execution from the negative byte

; Integer-Ausgabe 16 Bit in A (MSB), X (LSB) auf Screen
WriteInteger =	LAB_295E	; print XA as unsigned integer

; Integer-Ausgabe 16 Bit in A (MSB), X (LSB) auf COM-Kanal
SendInteger:
	STA FAC1_1	; save high byte as FAC1 mantissa1
	STX FAC1_2	; save low byte as FAC1 mantissa2
	LDA #$00
	STA FAC1_3	; clear FAC1 mantissa3
	LDX #$90	; set exponent to 16d bits
	STX FAC1_e	; set FAC1 exponent
	BNE SendIntegerFAC	; immer
	
; Bank * $10000 plus Startadresse bermitteln
SendBankStart:
	LDA CurrBank	; Bank-Adresse (*$10000) addieren
	STA FAC2_1
	LDA #0
	STA FAC2_2
	STA FAC2_3
	STA FAC2_s
	STA FAC1_r
	LDA #$98
	STA FAC2_e
	JSR LAB_2474	; add FAC2 to FAC1
SendIntegerFAC:
	SEC		; set integer is +ve flag
	LDA #$00
	STA FAC1_r	; clear FAC1 rounding byte
	STA FAC1_s	; clear FAC1 sign (b7)
	JSR LAB_24D0	; do ABS and normalise FAC1
	LDY #$00	; clear index
	TYA		; clear A
	JSR LAB_297B	; convert FAC1 to string, skip sign character save
	JSR LAB_20AE	; print " terminated string to Sutill/Sutilh
	JSR LAB_22B6	; pop string off descriptor stack, or from top of string
	LDY #$00	; reset index
	TAX		; copy length to X
	BEQ SendIntegerExit	; exit (RTS) if null string
SendIntegerLoop:
	LDA (ut1_pl),Y	; get next byte
	JSR SendByte	; go output the character
	INY		; increment index
	DEX		; decrement count
	BNE SendIntegerLoop	; loop if not done
SendIntegerExit:
	RTS

	
SendInpStr:		; 0-Terminated-String im Input Buffer an COM bermitteln
	LDX #0
SendInpStrLoop:
	LDA Ibuffs,x
	BEQ SendInpStrExit
	JSR SendByte
	INX
	BNE SendInpStrLoop
SendInpStrExit:
	RTS

; String-Expression in BASIC auswerten und in Ibuffs kopieren	
EvaString:
	JSR	LAB_EVEX
	JSR	LAB_EVST	; evaluate string
	
; String in Ibuf kopieren, Lnge in A
	TAX
	LDY #0
EvaStringLoop:
	LDA (ut1_pl),Y		; get byte from string
	STA Ibuffs,Y
	INY
	DEX
	BNE EvaStringLoop
	LDA #0
	STA Ibuffs,Y		; 0-Terminieren
	RTS
	
COMswitch:
	LDA COMCH
	STA COMCHsave
	LDA #0
	STA COMCH
	JSR LAB_FlushRx		; COM Buffer leeren
	RTS

;#####################################################################
;#                 OS-Routinen, LOAD, DIR und SAVE                   #
;#            und weitere c't-LAB-BASIC-Erweiterungen                #
;#####################################################################

LAB_LABOUT:				; PRINT to COM, -cm
	PHA
;	JSR LAB_FlushRx
	LDA #$80
	STA Omode
	PLA
	JMP LAB_PRINT1			; geht ber WriteAcc

; neue LABREQ-Routine mit erweiterter Fehlerbehandlung

LAB_LABREQ: 				; perform LABREQ mainch, subch, result
	JSR 	LAB_FlushRx
	JSR 	LAB_LABCMDPREP
	LDA	#'['			; fhrt zu Abbruch/String-Ende
	STA	LabReqIgnoreChr
	LDA 	#'?'	
	JSR 	SendByte
	LDA 	#$0D	
	JSR 	SendByte
	JSR 	LAB_SCOMMA		; scan for ","
	JSR	ReceiveString
	BCC	LABINP_timeout		; zu lange gedauert
	LDX	HashSignIdx
	BEQ	LABINP_falseResponse	; kein "#" empfangen, also ganz daneben
	
; berprfung: Mit LabReqMainCh und LabReqSubCh bereinstimmend?
	LDA	LabReqMainCh
	CMP	#9			; Wildcard-Channel FPGA?
	BEQ	LAB_LABREQ0		; dann keine berprfung auf MainCh
	LDA	Ibuffs,X		; nur ein Zeichen fr MainCh, deshalb simpel:
	SEC
	SBC	#'0'
	CMP	LabReqMainCh
	BNE	LABINP_wrongMainCh
LAB_LABREQ0:
	LDX	EqualSignIdx
	BEQ	LABINP_falseResponse	; kein "=" empfangen, also ganz daneben
	LDX	ColonSignIdx
	BEQ	LABINP_falseResponse	; kein ":" empfangen, also ganz daneben
	JSR	LAB_IBufIdx2Itemp	; String in Integer
	BIT	ItempH
	BMI	LABINP_falseResponse	; wenn kein SubCh empfangen
	CMP	#255			; Acc: war es der Fehler-Kanal?
	BNE	LAB_LABREQ1
	LDX	EqualSignIdx		; ja: Fehlercode zurckgeben
	JSR	LAB_IBufIdx2Itemp	; String in Integer, LSB in Acc
	STA	LabErrFlags
	JSR 	ErrStatus
	JMP	LAB_LABREQ2
LAB_LABREQ1:
	CMP	LabReqSubCh
	BNE	LABINP_wrongSubCh	; sollte gleich dem angeforderten sein
; MainCh und SubCh stimmen jetzt mit den angeforderten berein
LAB_LABREQ2:
	LDA	EqualSignIdx		; hier beginnt der eigentliche Wert
		
LAB_LABINP2:
	CLC
	ADC	#<Ibuffs
	TAX
	LDY	#>Ibuffs		; set Y to buffer start-1 high byte	
	JMP	LAB_BUFINP		; weiter wie INPUT
	
	
; returns timeout error
LABINP_timeout
	LDX	#'T'-'0'		; "Timeout" error
	BNE	LABINP_StoreError	; do LAB error #X 

; unexpected answer received - Falscher MainCh als Antwort
LABINP_wrongMainCh:
	LDX	#'M'-'0'
	BNE	LABINP_StoreError	; do LAB error #X 

; unexpected answer received - Falscher MainCh als Antwort
LABINP_wrongSubCh:
	LDX	#'S'-'0'
	BNE	LABINP_StoreError	; do LAB error #X 
	
; unexpected answer received - String unvollstndig oder verstmmelt, kein "#"
LABINP_falseResponse:
	LDX	#'F'-'0'		; "False response"
LABINP_StoreError:
	STX	LabErrFlags
	JSR 	ErrStatus

;	JMP MonitorBoot
	JMP	LAB_DATA		; scan for next BASIC statement ([:] or [EOL])
	RTS				; and leave variable unchanged
	
	
; String im Ibuffs+X in Integer wandeln, LSB in Acc
LAB_IBufIdx2Itemp:
	LDA	#$80
	STA	ItempH
	LDA	Bpntrl		; BASIC-Zeiger retten
	PHA
	LDA	Bpntrh
	PHA
	LDA	#>Ibuffs	; Pointer auf Buffer-Zeile
	STA	Bpntrh
	TXA			; war Index
	CLC
	ADC	#<IBuffs	; Startadresse addieren
	STA	Bpntrl
	JSR	LAB_GBYT	; Byte holen und prfen: Ziffer?
	BCS	LAB_IBufIdx2Itemp0	
; Falls keine Ziffer, berspringen und Syntax Error vermeiden
	JSR	LAB_GFPN
LAB_IBufIdx2Itemp0:
	PLA
	STA	Bpntrh		; BASIC-Zeiger wiederherstellen
	PLA
	STA	Bpntrl
	LDA	ItempL
; Ergebnis jetzt in Acc und ItempL/ItempH mit ItempH=$80 wenn Fehler
	RTS
	
	
LAB_LABINP:	; LABINP Routine fr ct-LAB-Antworten, kompletter String
	LDA	#$00
	STA	LabReqIgnoreChr 
	STA	LabErrFlags		; Clear LABBUS Error
	SEC				; String 
	JSR	ReceiveString
	BCC	LABINP_timeout
	LDA	#0
	JMP	LAB_LABINP2		; weiter wie READ

; LABERR may be checked in BASIC to recognize false readings
LAB_LABERR:
	LDY	LabErrFlags	
	JMP	LAB_1FD0	; convert Y to byte in FAC1 and return

	
;############################################################################

; LabReqMainCh	= $0713 ; angeforderter MainCh
; LabReqSubCh	= $0714 ; angeforderter SubCh

LAB_LABCMDprep:
	JSR 	LAB_GTBY	; get byte parameter Main Channel in X
	STX	LabReqMainCh
	LDA 	#0
	JSR 	SendInteger
	LDA 	#':'	
	JSR 	SendByte
	JSR 	LAB_SCGB	; scan for "," and get SubChannel in X
	STX	LabReqSubCh
	LDA 	#0
	JMP 	SendInteger	; returns with RTS

LAB_LABCMD:
	JSR 	LAB_LABCMDprep
	LDA 	#'='	
	JSR 	SendByte
	JSR 	LAB_SCOMMA	; scan for ","
	JMP 	LAB_LABOUT
	

	
;############################################################################
;BorderLeft 	=  $07D8 ; Textfenster 0..79, 0..40 2004 dez.
;BorderTop 	=  $07D9 ; 2005 dez.
;BorderRight	=  $07DA ; 2006 dez.
;BorderBtm 	=  $07DB ; 2007 dez.

BorderLinksOben:
	.BYTE ' ',198,214
BorderRechtsOben:
	.BYTE ' ',204,220
BorderRechtsUnten:
	.BYTE ' ',201,217
BorderLinksUnten:
	.BYTE ' ',195,211
BorderHor:
	.byte ' ',202,218
BorderVert:
	.byte ' ',197,213
	
LAB_WINDOW_BORDER:
	JSR	LAB_SGBY	; get byte parameter
	STX	work
	LDA	BorderLinksOben,X
	LDX	BorderTop
	LDY	BorderLeft
	JSR	writecharxy
	LDY	work
	LDA	BorderRechtsOben,Y
	LDY	BorderRight
	JSR	writecharxy
	LDX	work
	LDA	BorderRechtsUnten,X
	LDX	BorderBtm
	JSR	writecharxy
	LDY	work
	LDA	BorderLinksUnten,Y
	LDY	BorderLeft
	JSR	writecharxy
	JSR	LAB_WINDOW_BORDER_hor
		
LAB_WINDOW_BORDER_vert:	
	LDX	work
	LDA	BorderVert,X	
	LDX	BorderTop
	INX
LAB_WINDOW_BORDER_vert_loop:
	LDY	BorderLeft
	JSR	writecharxy
	LDY	BorderRight
	JSR	writecharxy
	INX
	CPX	BorderBtm
	BNE	LAB_WINDOW_BORDER_vert_loop	
	RTS
	
LAB_WINDOW_BORDER_hor:
	LDY	work
	LDA	BorderHor,Y	
	LDY	BorderLeft
	INY
LAB_WINDOW_BORDER_hor_loop:
	LDX	BorderBtm
	JSR	writecharxy
	LDX	BorderTop
	JSR	writecharxy
	INY
	CPY	BorderRight
	BNE	LAB_WINDOW_BORDER_hor_loop	
	RTS

LAB_WINDOW:
	CMP	#TK_CLEAR	; compare with token for CLEAR
	BEQ	LAB_WINDOW_CLEAR
	CMP	#TK_CURSXY	; compare with token for COLOR
	BEQ	LAB_WINDOW_CURSOR
	CMP	#TK_POSITION	; compare with token for POSition
	BEQ	LAB_WINDOW_POS
	CMP	#TK_BORDER	; compare with token for BORDER
	BEQ	LAB_WINDOW_BORDER
	CMP	#TK_OFF	; compare with token for OFF
	BEQ	LAB_WINDOW_OFF
	RTS	; Error
	
LAB_WINDOW_OFF
	JSR	LAB_IGBY	; just increment
	JMP 	SetFullScreen
	
LAB_WINDOW_POS:
	JSR	LAB_SGBY	; get byte parameter
	STX	TextWinLeft
	STX	col
	STX	CursX
	DEX
	STX	BorderLeft
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	TextWinTop
	STX	row
	STX	CursY
	DEX
	STX	BorderTop
	LDA	#TK_TO		; get "TO" token
	JSR	LAB_SCCA	; scan for CHR$(A) , else do syntax error then warm start
	JSR	LAB_GTBY	; get byte parameter
	STX	TextWinRight
	INX
	STX	BorderRight
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	TextWinBtm
	INX
	STX	BorderBtm
	RTS
	
LAB_WINDOW_CLEAR:
	JSR	LAB_IGBY	; just increment
	JMP	ClearWindow

	
LAB_WINDOW_CURSOR:		; Relative Cursor-Position innerhalb Fenster
	JSR	LAB_SGBY	; get byte parameter
	TXA
	CLC
	ADC	TextWinLeft
	STA	col
	STA	CursX
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	TXA
	CLC
	ADC	TextWinTop
	STA	row
	STA	CursY
	RTS
	
	
LAB_COLOR:
	JSR	LAB_GTBY	; get byte parameter
	STX	Color
	STX	ColorP
	RTS

LAB_LABBUS:
	JSR	LAB_GTBY	; get byte parameter
	CPX	#$03		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
	STX	COMCH
	JSR	LAB_FlushRx
	STA 	UART1TxC	; auch UART-TxFIFO leeren
	TXA
LAB_LABBUS_SEL:	
	CMP	#$02		; LABBUS 2 ist externes RS-232
	BNE	LAB_LABBUS1
	LDA	#1
	STA	UART_SEL	; Multiplexer fr ser. Schnittstelle
	JSR 	ComStatus
	RTS
LAB_LABBUS1:	
	LDA	#0
	STA	UART_BAUD
	STA	UART_SEL
	JSR 	ComStatus
	RTS			; return to BASIC

LAB_CURSXY:
	JSR	LAB_GTBY	; get byte parameter
	STX	cursx
	STX	col
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	cursy
	STX	row
	RTS			; return to BASIC
; does BASIC function call error
LAB_CURSXY_FCER
	JMP	LAB_FCER	; do function call error, then warm start

SyntaxErr1:
	JMP	LAB_SNER	; do syntax error, then warm start

; Select Memory BANK command -cm
LAB_BANK
	JSR	LAB_GTBY	; get byte parameter
	CPX	#$08		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
	STX	Bank		; set new BANK
	STX	CurrBank
	STX	AccessBank
	JSR Bankstatus
	JMP PScanSetBasParam	; BASIC-Parameter neu setzen, wie nach LOAD
	
LAB_FlushRx:		; Buffer leeren, benutzt nur Acc
	SEI
	STA 	UART1RxC	; UART-RxFIFO leeren
LAB_FlushRx0:		; Buffer leeren, benutzt nur Acc
	LDA	UART1D
	BIT	UART1RxS
	BMI	LAB_FlushRx0
	LDA	#$00
	STA	LabReqIgnoreChr 
	STA 	iRxPtr		; Input Buffer leeren
	STA 	RxPtr
	LDA	#0
	STA	LabErrFlags	; Clear LABBUS Error
	JSR 	ErrStatus
	CLI
	RTS

LAB_SCANKEY:
	LDY	#0
	LDA	peekEvent	; neuer Event aufgetreten?
	BEQ	LAB_SCANKEY_exit
	LDY	peekScancode	; ja, Code zurckgeben
LAB_SCANKEY_exit	
	LDA	#0
	STA	peekEvent	
	JMP	LAB_1FD0	; convert Y to byte in FAC1 and return


;############################################################################

;Grafik-Routinen
;Gsmall		= $0327	; Grafik Pixelgre, 0=DoubleScan (gro), 1=klein
;GstartX	= $0328	; Grafik-Fenster Startposition X
;GstartY	= $0329	; Grafik-Fenster Startposition Y
;GsizeX		= $032A	; Grafik-Fenster Gre X
;GsizeY		= $032B	; Grafik-Fenster Gre Y
;GridX		= $032C	; Grafik-Fenster Gitterraster Abstand X
;GridY		= $032D	; Grafik-Fenster Gitterraster Abstand Y
;GFrameC	= $032E	; Grafik-Fenster Hintergrundfarbe 0..7, 8..15 mit Raster und Rand
;GcolorP	= $032F	; Grafik-Fenster Punkt- und Linienfarbe

LAB_FRAME:
	CMP	#TK_CLEAR	; compare with token for CLEAR
	BEQ	LAB_FRAME_CLEAR
	CMP	#TK_COLOR	; compare with token for COLOR
	BEQ	LAB_FRAME_COLOR
	CMP	#TK_POSITION		; compare with token for POSition
	BEQ	LAB_FRAME_POS
	CMP	#TK_WIDTH	; compare with token for WIDTH
	BEQ	LAB_FRAME_WIDTH
	CMP	#TK_GRID	; compare with token for GRID
	BEQ	LAB_FRAME_GRID
	CMP	#TK_SIZE	; compare with token for SIZE
	BEQ	LAB_FRAME_SIZE
	RTS	; Error
	
LAB_FRAME_SIZE:
; Grafik-Fenster lschen
	JSR	LAB_SGBY	; Size parameter
	CPX	#$02		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
	DEX
	STX	Gsmall		; Grafik Pixelgre, 0=DoubleScan (gro), 1=klein
	RTS
	
LAB_FRAME_COLOR:
; Grafik-Fenster lschen
	JSR	LAB_SGBY	; Bkgns color parameter
	CPX	#$20		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
	STX	GFrameC
	RTS
		
LAB_FRAME_CLEAR:
; Grafik-Fenster lschen
	JSR 	LAB_IGBY
FrameClear:
;ein Rechteck in Frame-Gre mit Null fllen, sync'd mit PLOTSYNC
	BIT 	syncFlgs	; frame generated?
	BVC 	FrameClear	; ..no, wait for frame start
FcwaitL:			; wait for PLOTSYNC completed
	BIT 	syncFlgs	; frame generated?
	BVS 	FcwaitL		; ..no, wait for frame end

	LDX	#0
	STX	GraphX
	STX	GraphY
	STX	GcolorP
	STX	GmvTrig		; MOVETO
	DEX
	STX	GraphX
	LDX	#$BF
	STX	GraphY
	STX	GflTrig		; FILLRECT
	NOP
LAB_FRAMEClear2	
	BIT	GDone		; Grafik fertig?
	BPL	LAB_FRAMEClear2
LAB_FRAMEClear3	
	RTS
	
	
LAB_FRAME_POS:
; Fensterposition setzen
	JSR	LAB_SGBY	; get byte parameter
	STX	GstartX		; Grafik-Fenster Startposition X
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	GstartY
	RTS
	
LAB_FRAME_WIDTH:
; Fensterposition setzen
	JSR	LAB_SGBY	; get byte parameter
	STX	GsizeX		; Grafik-Fenster Startposition X
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	GsizeY
	RTS
	
LAB_FRAME_GRID:
; Fensterposition setzen
	JSR	LAB_SGBY	; get byte parameter
	STX	GridX
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	GridY
	RTS

;GraphX		= $0320	; X-Koordinatenregister, 0..255, oben links = 0,0
;GraphY		= $0321	; Y-Koordinatenregister, 0..255, oben links = 0,0
;GmvTrig	= $0322	; PLOT Move-Befehl triggern, Daten ignoriert, beliebiger Schreibzugriff
;GptTrig	= $0323 ; PLOT Point-Befehl triggern, Daten ignoriert
;GlnTrig	= $0324	; PLOT Line-Befehl triggern, Daten ignoriert
	
LAB_PLOT:
	CMP	#TK_COLOR	; compare with token for CLEAR
	BEQ	LAB_PLOT_COLOR
	CMP	#TK_FILL+1	; more than FILL?
	BCS	LAB_FCER2	; if greater do function call error
	SEC
	SBC	#TK_MOVE
	STA	asave
	CLC
	JSR	LAB_SGBY	; get byte parameter in X
LAB_PLOT1:	; Plot-Modus (MOVE,PIXEL,LINE,RECT,FILL) 0..4 in asave
	BIT	GDone		; Grafik fertig?
	BPL	LAB_PLOT1
	LDA	LastPlotColor	; letzte Farbe, falls "Cera" erase	
	STA	GColorP
	STX	GraphX
	STX	LastPlotX
	JSR	LAB_SCGB	; scan for "," and get byte parameter in X
	STX	GraphY
	STX	LastPlotY
	LDX	asave	
	STA	GmvTrig,X	; X > 0=MOVE, 1=POINT, 2=LINE, 3=RECT, 4=FILL
	RTS
LAB_PLOT_COLOR:	; Pinselfarbe setzen
	BIT	GDone		; Grafik fertig?
	BPL	LAB_PLOT_COLOR
	JSR	LAB_SGBY	; Bkgns color parameter
	CPX	#$20		; compare with max
	BCS	LAB_FCER2	; if greater do function call error
	STX	GColorP
	STX	LastPlotColor
	RTS
	
LAB_FCER2:
	JMP	LAB_FCER	; do function call error, then warm start
	
SyntaxErr2:
	JMP	LAB_SNER	; do syntax error, then warm start
	


;############################################################################

LAB_CardDir:	; Directory on Screen
	JSR COMswitch
	JSR SendMsg
	.ASCII "9:DIR",$0D
	NOP
	JMP ResponseAndExit	; do EhBASIC warm start

LoadSavePrep:
; Default-AutoIncrement-Parameter setzen
	LDA iKbdPtr		; Kbd-buffer lschen
	STA KbdPtr

	JSR SendMsg
	.ASCII "9:AIS=0",$0D
	.ASCII "9:AIR=128",$0D	; Default-AutoIncRegister
	.ASCII "9:AIW=1",$0D
	.ASCII "9:AIM="	; Startadresse bermitteln
	NOP
	RTS
	
AccToFAC1:
	STA FAC1_1
	LDA #0
	STA FAC1_2
	STA FAC1_3
	LDA #$90	; Exponent fr 24 Bit
	STA FAC1_e
	RTS
	
LAB_SaveFile:
	JSR EvaString	; String in " " in IBuffs kopieren
	JSR COMswitch
	DEY
	BEQ SyntaxErr2
	JSR LoadSavePrep
	LDA smemh	; FAC1 mit BASIC-Startadresse $0800 vorbesetzen
	JSR AccToFAC1
	JSR SendBankStart
	JSR pscan	; Ende feststellen
	ldy #$00
	lda itempl
	sta (itempl),y
	iny
	lda itemph
	sta (itempl),y
	JSR SendMsg
	.ASCII $0D,"9:AIE="
	NOP
	INC itemph		; Page +1
	LDA itemph
	JSR AccToFAC1
	JSR SendBankStart
	JSR SendMsg	
	.ASCII $0D,"9:BSV=",34
	NOP
	JSR SendInpStr		; evaluated string
	JSR LAB_FlushRx
	JSR SendMsg	
	.ASCII 34,'!',$0D
	NOP
	LDA #$80	; ACK einschalten
	STA Tx0ACK
	
ResponseAndExit:
	JSR WriteResponse
NoResponseAndExit:
	JSR LAB_FlushRx
	LDA COMCHsave
	STA COMCH
	LDA CurrBank
	STA Bank
	JMP LAB_WARM	; do EhBASIC warm start
	
LAB_LoadFile		; load vector for EhBASIC
	JSR EvaString	; String in " " in IBuffs kopieren
	JSR LAB_GBYT
	CMP #',' 	; compare with comma
	BNE LoadnoParam
	JSR LAB_IGBY
	JSR LAB_EVNM	; evaluate expression and check is numeric, put into FAC1,
			; else do type mismatch
	JMP LoadFile	; nur laden
	
LoadnoParam:
	LDA smemh	; FAC1 mit BASIC-Startadresse $0800 vorbesetzen
	JSR AccToFAC1
	JSR LoadFile	
	JMP PScanSetBasParam	; BASIC-Parameter neu setzen

LoadFile:
	JSR COMswitch
	DEY
	BEQ SyntaxErr3
	JSR LoadSavePrep
; Integer-Ausgabe 24 Bit in CurrBank, Itempl/Itemph auf COM-Kanal
	JSR SendBankStart
	JSR SendMsg	
	.ASCII $0D,"9:BLD=",34 ; 
	NOP
	JSR SendInpStr		; evaluated string
	LDA	#5
	JSR 	TickerDelay
	JSR LAB_FlushRx
	JSR SendMsg	
	.ASCII 34,'!',$0D
	NOP
	LDA #$80	; FlowControl einschalten
	STA Tx0ACK
	JSR WriteResponse
	LDA CurrBank
	STA Bank
	LDA COMCHsave
	STA COMCH
	RTS
		
SyntaxErr3:
	JMP	LAB_SNER	; do syntax error, then warm start


LAB_MACRO:	; Type Textfile to SPI Kbd or execute .INI
	JSR EvaString	; String in " " in IBuffs kopieren
	JSR COMswitch	; String-Ende +1 jetzt in Y
	DEY
	BEQ SyntaxErr3
	TYA
	PHA 

	LDA iKbdPtr		; Kbd-buffer lschen
	STA KbdPtr
	JSR LAB_FlushRx

	JSR SendMsg	
	.ASCII "9:FQU=",34 	; File vorhanden?
	NOP
	JSR SendInpStr		; evaluated string
	JSR SendMsg	
	.ASCII 34,'!',$0D
	NOP
	JSR WriteResponse
	PLA
	TAY
	LDX #3			; Stringlnge-1	
LAB_MACRO_Compare:
	LDA	INIstring,Y
	CMP 	IbuffS,X 
	BNE 	TypeFile
	DEY
	DEX
	BPL	LAB_MACRO_Compare	
; muss jetzt Endung .INI haben
	JSR SendMsg	
	.ASCII "9:CFG=",34 ; 
	NOP
	JSR SendInpStr		; evaluated string
	JSR SendMsg	
	.ASCII 34,$0D
	NOP
	JMP NoResponseAndExit	; do EhBASIC warm start
	
TypeFile:
	JSR SendMsg
	.ASCII "9:TTF 67=",34 	; 
	NOP
	JSR SendInpStr		; evaluated string
	JSR SendMsg	
	.ASCII 34,$0D
	NOP
	JMP NoResponseAndExit	; do EhBASIC warm start

INIstring:
	.DB ".INI" 
	
;############################################################################
	
PScan	; Hilfsroutine: Suche nach Programm-Ende
	lda	smeml
      	sta	itempl
      	lda	smemh
      	sta	itemph
Pscan1	ldy	#$00
	lda	(itempl),y
	bne	pscan2
	iny   
	lda	(itempl),y
	bne	pscan2
	clc
	lda	#$02
	adc	itempl
	sta	itempl
	lda	#$00
	adc	itemph
	sta	itemph
	rts
Pscan2	ldy	#$00
	lda	(itempl),y
	tax
	iny
	lda	(itempl),y
	sta	itemph
	stx	itempl
	JMP	pscan1
	
PScanSetBasParam:
	JSR Pscan	; BASIC-Parameter neu setzen
	LDA itempl
	STA svarl
	STA sarryl
	STA earryl
	LDA itemph
	STA svarh
	STA sarryh
	STA earryh
	
	LDA RAM_base+5	; erster Zeile "REM AUTOSTART"?
	CMP #TK_REM
	BNE noAutoStart
	LDY #7		; Q&D: Absolute Adresse
	LDX #0
AutoSearchLoop:	
	LDA RAM_base,Y
	STA asave
	LDA AutoStartString,X
	CMP #0		; alles verglichen, Ende-Kennung
	BEQ AutoStartRun
	INY
	INX
	CMP asave
	BEQ AutoSearchLoop

	LDX #0
	JMP  LAB_warm			; 
	
AutoStartRun:
	LDA	#15	; 0,25s
	JSR	TickerDelay
	JMP  LAB_1477	; Autostart-RUN	
noAutoStart:
	JMP  LAB_warm			; Warmstart
;	JMP  LAB_1319			; Warmstart ohne "Ready"
AutoStartString:
	.DB "AUTOSTART",00  
	

;############################################################################
;############################################################################
;#        OS-Routinen, RST, IRQ-Aufbereitung und Initialisierung            #
;############################################################################
;############################################################################


Reset:
	SEI		; IRQ sperren bis IRQ-Routine ins RAM kopiert
	CLD		; clear decimal mode
	
	LDX #$FF	; empty stack
	TXS		; set the stack
	
	LDA	#<WriteAcc
	STA	WritejpL	; alten Write-Vektor wiederherstellen
	LDA	#>WriteAcc
	STA	WritejpH
	LDA	#<GetKey
	STA	GetkeyjpL	; alten GetKey-Vektor wiederherstellen
	LDA	#>GetKey
	STA	GetkeyjpH

	LDA #0
	STA COMCH
	STA IRQena
	STA iRxPtr
	STA RxPtr
	STA KbdPtr
	STA iKbdPtr
	STA CapsFlg	; UpCase-Flag, $20=Basic, $00=Terminal
	STA lastkey
	STA GetbFlg
	STA Omode	; 0 wenn PRINT, $80 wenn LABOUT auf COM
	STA Gsmall
	STA GFrameC	; Fenster-Hintergrund
	LDA IRQack
	LDA IRQclr
	LDA CurrBank
	STA Bank
	STA AccessBank	; fr PEEKs/POKEs
	LDA #$80	; FlowControl einschalten
	STA Tx0ACK

	STA GsizeX
	LDA #50		; Grafik-Fenster
	STA GstartX
	LDA #36		; Grafik-Fenster
	STA GstartY
	LDA #$60	; Grafik-Fenster
	STA GsizeY
	
	JSR ClearFullScreen
	
	JSR CapsToggle
	JSR COMstatus
	LDA #35		; Repeat-Timer neu setzen
	STA repeat
	LDA #5
	STA repRate
	STA Timer60
	CLI
	
; now do the signon message, Y = $00 here

LAB_signon
	.IF RealWorld
	LDA #10
	JSR	TickerDelay
	JSR COMswitch
	LDA #$0D
	JSR SendByte
	LDA #5
	JSR	TickerDelay
	JSR LAB_FlushRx
	LDA COMCHsave
	STA COMCH
	
	.endif
	JSR LAB_FlushRx
	JSR WriteMsg
	.BYTE $0D,$0D,$0A,"[C]old/[W]arm/[T]erm/[M]on ?"	
	NOP
LAB_nokey
	JSR GetKey		; call scan input device
	BCC LAB_nokey		; loop if no key
	AND #$DF		; mask xx0x xxxx, ensure upper case
	CMP #'W'		; compare with [W]arm start
	BEQ LAB_dowarm		; branch if [W]arm start
	CMP #'T'		; compare with [T]erminal
	BEQ TermJmp
	CMP #'M'		; compare with [M]oitor
	BEQ MonJmp
 	CMP #'C'		; compare with [C]old start
	BNE LAB_again		; loop if not [C]old start
LAB_release1
	JSR GetKey		; call scan input device
	BCS LAB_release1	; loop if key
	LDA #0
	LDY #$07
ResetPageLoop:
	STY Bank
	STA RAM_base
	STA RAM_base+1
	STA RAM_base+2
	DEY
	BPL ResetPageLoop
	STA CurrBank
	STA AccessBank
	JSR Bankstatus
	JMP LAB_COLD		; do EhBASIC cold start
LAB_dowarm
	JSR Bankstatus
	JSR WriteScreen
LAB_release2
	JSR GetKey		; call scan input device
	BCS LAB_release2	; loop if key
	JMP LAB_WARM		; do EhBASIC warm start
TermJmp JSR Terminal
	JMP Reset	
MonJmp	JSR MonitorBoot
LAB_again:
	JMP Reset

;############################################################################
;#                            IRQ- und BRK-Handler                          #
;############################################################################

; BRK handler: Register retten und Monitor aufrufen

IRQentry:
	STA irqa	; hier mal nicht auf Stack
	LDA IRQsrc	; HW-Interrupt-Quelle vorrangig
	AND #$7F	; GDONE ausblenden
	STA irqs	; retten	
	BNE IRQnoBRK	; HW hat Vorrang
	PLA		; Get status
	PHA		; Re-save status
	AND #$10	; Is it IRQ or BRK ?
	BEQ IRQnoBRK
BRKentry:		; BRK Handler
	LDA irqa
	STA ACC		; fr Monitor
	STX Xreg	; save X for monitor
	STY Yreg	; save Y	
 	LDY IRQack
	LDY IRQclr	; IRQsrc gelesen, Latch lschen
	pla		; 
	STA Preg	; save P status
	PLA		; PCL
	STA PCL
	PLA		; PCH
	STA PCH
	TAX
	LDA PCL		; PCL
	SEC		; 2 subtrahieren fr echte BRK-Adresse
	SBC #$02	;
	STA PCL		; backup to BRK cmd
	BCS Brk2	;
	DEX		;
Brk2:	STX PCH		; save PC
	LDA irqa	; geretteter Acc
	TSX		; get stack pointer
	STX SPtr	; save stack pointer
	JSR PrintReg	; dump register contents
	LDX #$FF	; 
	TXS		; clear stack
	CLI		; enable interrupts again
	PLP
	PHP
	JMP Monitor	; start the monitor
	RTI
	
; CMs IRQ Handler: COM-, VS60Hz- und SPI-IRQ auswerten

IRQnoBRK:
	LDA irqa
	PHA
	TYA
	PHA
	LDY IRQack	; IRQ Reset
	LDY IRQclr	; IRQsrc gelesen, Latch lschen
; IRQs zuordnen: Bit 0 (SPI Strobe 0) nicht belegt
	LDA irqs	; Saved IRQsrc
	AND #$04	; Bit 2, Received Byte auf UART?
	BNE IRQ_UART
	LDA irqs
	AND #$02	; Bit 1, Received Byte auf AVRCOM?
	BEQ IRQ_KBD	; keines von beiden, weiter mit Kbd
	
; war Bit 1, also AVRCOM
IRQ_COM:
	LDA COMport	; COM-Byte von SPI eingetroffen
	BEQ IRQ_KBD	; Null-byte, nix machen
	CMP #$06
	BNE IRQ_COM_store
	LDA #$80	; ACK feststellen, aber nicht speichern
	STA Tx0ACK
	BNE IRQ_KBD	; immer!
	
; war Bit 2, also UART
IRQ_UART:
	BIT UART1RxS	; Rx Status, Bit 7 = data present
	BPL IRQ_KBD	; Bit 7 = "0", nix mehr da
	LDA UART1D	; Byte abholen, setzt auch Rx_ack
	LDY iRxPtr	; war AVRCOM oder UART
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA RxBuf,Y	; in Eingangspuffer
	STY iRxPtr
	JMP IRQ_UART	; noch was da?
;	JMP IRQ_KBD
	
IRQ_COM_store:
	LDY iRxPtr	; war AVRCOM oder UART
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA RxBuf,Y	; in Eingangspuffer
	STY iRxPtr
IRQ_KBD:
	LDA irqs
	AND #$08	; SPI Keystrobe?
	BEQ IRQexit		
	LDY iKbdPtr	; war SPI-Key
	LDA SPIkey	; Keyboard-Byte von SPI eingetroffen
	BEQ IRQexit	; Null-byte, nix machen
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA KbdBuf,Y
	STY iKbdPtr	
IRQexit:
	PLA
	TAY
	PLA
	RTI
	
; 60Hz Ticker NMI, reentrant zu IRQ
NMIentry:
	PHA
;	LDA NMIack	; NMI angenommen
	LDA TimeOut	
	BEQ NMInoDecTimer
	DEC TimeOut
NMInoDecTimer
	DEC Timer60	; war VSync
NMInoSecond:	
	PLA
	RTI

statusline1:
	.IF LCD320
	.db "EhBASIC #2.2c BANK   LABBUS"
	.DB 0	; String-Ende, max. 39 Zeichen!
	.ELSE
	.db "c't-Lab EhBASIC Terminal #2.2c by cm        "  
	.db "BANK    LABERR    LABBUS"
	.DB 0	; String-Ende, max. 79 Zeichen!
	.ENDIF

	
HelpTxt        .BYTE $0d, $0a
               .byte "x = HEX 4 char max~"
               .byte "x.x - Hex dump~"
               .byte "x:DD[ DD] - Change bytes~"
               .byte "xG - Go~"
               .byte "xL - List code~"
               .byte "x.x>xI - Move down~"
               .BYTE "x.x>xM - Move up~"
               .byte "x.xQ - Text dump~"
               .byte "R - Regs~"
               .byte "V - Version~"
               .byte "X - Exit~"
               .byte $00
Porttxt        .byte "6502 Mon v5.1"
               .byte $0d, $0a
               .byte "(? for help)"
               .byte $00

	*= $FFF4		; CALL -12 Terminal, -9 Monitor
	JMP Terminal
	JMP MonitorBoot
	
; system vectors
	*= $FFFA
SysVec:
	.word	NMIentry	; NMI vector
	.word	Reset		; RESET vector
	.word	IRQentry	; IRQ vector


	.END
	
; UNUSED BASIC FUNCTIONS
; perform CONT, replaced by graph function by -cm

	.IF LCD320
	
rowshi:		; Anfangsadressen der Bildschirmzeilen 40 Zeichen LCD, 20/25 Zeilen
	.DB >VRAM, >VRAM, >VRAM, >VRAM, >VRAM, >VRAM, >VRAM
	.DB >VRAM+1, >VRAM+1, >VRAM+1, >VRAM+1, >VRAM+1, >VRAM+1 
	.DB >VRAM+2, >VRAM+2, >VRAM+2, >VRAM+2, >VRAM+2, >VRAM+2, >VRAM+2
	.DB >VRAM+3, >VRAM+3, >VRAM+3, >VRAM+3, >VRAM+3, >VRAM+3
rowslo:
	.DB 0, 40, 80, 120, 160, 200, 240, 24, 64, 104, 144, 184, 224
	.DB 8, 48, 88, 128, 168, 208, 248, 32, 72, 112, 152, 192, 232
	
	.ELSE
rowshi:		; Anfangsadressen der Bildschirmzeilen 80 Zeichen VGA
	.DB >VRAM, >VRAM, >VRAM, >VRAM, >VRAM+1, >VRAM+1, >VRAM+1
	.DB >VRAM+2, >VRAM+2, >VRAM+2, >VRAM+3
	.DB >VRAM+3, >VRAM+3, >VRAM+4, >VRAM+4, >VRAM+4, >VRAM+5
	.DB >VRAM+5, >VRAM+5, >VRAM+5, >VRAM+6, >VRAM+6
	.DB >VRAM+6, >VRAM+7, >VRAM+7, >VRAM+7, >VRAM+8
	.DB >VRAM+8, >VRAM+8, >VRAM+9, >VRAM+9, >VRAM+9, >VRAM+10
	.DB >VRAM+10, >VRAM+10, >VRAM+10, >VRAM+11, >VRAM+11
	.DB >VRAM+11, >VRAM+12
rowslo:
	.DB 0, 80, 160, 240, 64, 144, 224, 48, 128, 208, 32, 112, 192
	.DB 16, 96, 176, 0, 80, 160, 240, 64, 144, 224, 48
	.DB 128, 208, 32, 112, 192, 16, 96, 176, 0, 80, 160
	.DB 240, 64, 144, 224, 48
	
	.ENDIF

LAB_CONT
	BNE	LAB_167A		; if following byte exit to do syntax error
	LDY	Cpntrh			; get continue pointer high byte
	BNE	LAB_166C		; go do continue if we can

	LDX	#$1E			; error code $1E ("Can't continue" error)
	JMP	LAB_XERR		; do error #X, then warm start

					; we can continue so ..
LAB_166C
	STY	Bpntrh		; save BASIC execute pointer high byte
	LDA	Cpntrl		; get continue pointer low byte
	STA	Bpntrl		; save BASIC execute pointer low byte
	LDA	Blinel		; get break line low byte
	LDY	Blineh		; get break line high byte
	STA	Clinel		; set current line low byte
	STY	Clineh		; set current line high byte
	RTS

	

	
LAB_SubCh2accold:
	LDA	
	LDX	EqualSignIdx
	DEX
	DEX			; zeigt jetzt auf letzte Ziffer
LAB_SubCh2accLoop:
	LDA	Ibuffs,X	; Einer
	SEC
	SBC	#'0'
	STA	asave
	CPX	ColonSignIdx
	BEQ	LAB_SubCh2accDone
	DEX
	LDA	Ibuffs,X	; Zehner
	JSR	mult10
	CLC
	ADC	asave
	STA	asave
	CPX	ColonSignIdx
	BEQ	LAB_SubCh2accDone
	DEX
	LDA	Ibuffs,X	; Hunderter
	JSR	mult10
	JSR	mult10_0
	CLC
	ADC	asave
LAB_SubCh2accDone:
	RTS

MULT10:
	SEC
	SBC	#'0'
MULT10_0:
	ASL		;multiply by 2
	STA work	;temp store
	ASL		;again multiply by 2 (*4)
	ASL		;again multiply by 2 (*8)
	CLC
	ADC work	;as result, A = x*8 + x*2
	RTS

	.ENDIF
